SystemOrganization addCategory: #MECCG!
SystemOrganization addCategory: 'MECCG-Model'!
SystemOrganization addCategory: 'MECCG-Model-Tests'!
SystemOrganization addCategory: 'MECCG-Support'!
SystemOrganization addCategory: 'MECCG-Support-Tests'!

TestCase subclass: #MECCGBoardTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model-Tests'!

!MECCGBoardTest commentStamp: '<historical>' prior: 0!
A MECCGBoardTest is a test class for testing the behavior of MECCGBoard!

!MECCGBoardTest methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 14:51'!
test_01_AssignCorrectBoardToMatch

	| match |
	match := Match of: MiddleEarthCardGame rules.
	match board: MECCGBoard new.
	! !

!MECCGBoardTest methodsFor: 'public' stamp: 'FedericoFigueredo 5/9/2016 09:09'!
test_04_BoardShouldHaveOneZoneForEachPlayer
	| board | 
	board := MECCGBoard new.
	
	self assert: (MECCGBoardValidator new isAmountOfZonesValidIn: board).
	! !

!MECCGBoardTest methodsFor: 'public' stamp: 'FedericoFigueredo 5/9/2016 09:21'!
test_05_PlayerZoneShouldHave2Decks
	| board | 
	board := MECCGBoard new.
	
	self assert: (MECCGBoardValidator new isAmountOfDecksValidIn: board).! !

!MECCGBoardTest methodsFor: 'as yet unclassified' stamp: 'FedericoFigueredo 5/9/2016 09:22'!
test_06_PlayerZoneShouldHave3Piles
	| board | 
	board := MECCGBoard new.
	
	self assert: (MECCGBoardValidator new isAmountOfPilesValidIn: board).
	! !

!MECCGBoardTest methodsFor: 'as yet unclassified' stamp: 'FedericoFigueredo 5/9/2016 09:23'!
test_07_PlayerZoneShouldHave1Hand
	| board | 
	board := MECCGBoard new.
	
	self assert: (MECCGBoardValidator new isAmountOfHandsValidIn: board).
	! !

!MECCGBoardTest methodsFor: 'as yet unclassified' stamp: 'FedericoFigueredo 5/9/2016 09:24'!
test_08_PlayerZoneShouldHaveSomewhereToPlayCards
	"Fede, I expect you to (define/find the definition of) this zone"

	self assert: false.
	! !

TestCase subclass: #MECCGLoggerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support-Tests'!

!MECCGLoggerTest commentStamp: '<historical>' prior: 0!
A MECCGLoggerTest is a test class for testing the behavior of MECCGLogger!

!MECCGLoggerTest methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 14:43'!
test_01_Subscribing

	| logger |
	logger := MECCGLogger on: self.
	self assert: logger source = self.
	self deny: (self hasActionsWithReceiver: logger).
	logger logEvents: #testEvent.
	self assert: (self hasActionsWithReceiver: logger).
	logger detach.
	self deny: (self hasActionsWithReceiver: logger).
! !

!MECCGLoggerTest methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 14:39'!
test_02_LogIsEmptyAtCreation

	| logger |
	logger := MECCGLogger on: self.
	self assert: (logger log isEmpty).
! !

!MECCGLoggerTest methodsFor: 'public' stamp: 'FedericoFigueredo 5/8/2016 14:02'!
test_03_EventsAreAddedToLog

	| logger |
	logger := MECCGLogger on: self.
	self assert: (logger log isEmpty).
	logger logEvents: #testEvent.
	self assert: (logger log isEmpty).
	self triggerEvent: #testEvent with: 'Test message'.
	self assert: (logger log isEmpty not).
	self assert: (logger log isSubstringOfInfiniteLoopGeneratedBy: 'Test message').
	logger detach.
	self assert: (logger log isEmpty not).
	self deny: (self hasActionsWithReceiver: logger).
! !

TestCase subclass: #MECCGModelUsageExamples
	instanceVariableNames: 'match logger players'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model-Tests'!

!MECCGModelUsageExamples commentStamp: 'SergioFedi 5/7/2016 11:33' prior: 0!
This class holds various integration and functional tests for the model.

The first test shows a typical usage and how to setup a logger that registers the different activities that take place in the match.
!

!MECCGModelUsageExamples methodsFor: 'private' stamp: 'FedericoFigueredo 5/6/2016 09:29'!
checkLogger: logger toMakeSureThePhaseOrderIsTheSameAsIn: phases 
	| logMessages |
	logMessages  := OrderedCollection new.
	phases do: [ :phase | 
		logMessages add: 'Begin phase named ', phase name.
		logMessages add: 'End of phase named ', phase name.
	].
	self assert: (logger areTheseMessagesOrdered: logMessages).! !

!MECCGModelUsageExamples methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 12:01'!
checkLogger: logger toMakeSureThePlayerOrderIsTheSameAsIn: players 
	| logMessages |
	logMessages  := players collect: [ :player | 
		'Begin turn of player ', player name ].
	self assert: logger logSize >= players size.
	self assert: (logger areTheseMessagesOrdered: logMessages).! !

!MECCGModelUsageExamples methodsFor: 'private' stamp: 'SergioFedi 5/6/2016 14:03'!
checkThatLastActionPlayedInEachPhaseIsEndPhaseInLogger: logger 

	| beginningsOfLogMessages |
	beginningsOfLogMessages  := Array 
		with: 'Begin phase'
		with: 'Action generic'
		with: 'Action EndPhase '
		with: 'End of phase'.
	self assert:
		(logger 
			areTheseMessagesOrdered: beginningsOfLogMessages 
			allowingMultipleInstancesOf: beginningsOfLogMessages second).! !

!MECCGModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/10/2016 11:47'!
playMatch
	"Initialize the test variables and play a standard match"

	match := Match of: MiddleEarthCardGame rules.
	logger := MECCGLogger on: match.
	players := OrderedCollection
		with: (MECCGPlayer named: 'Federico')
		with: (MECCGPlayer named: 'Sergio').
	logger logEvents: #somethingHappened.
	match addPlayers: players.
	match board: MECCGBoard new.
	self assert: match isReadyForSetup.
	match setup.
	self assert: match isReadyForPlay.
	match play.
	match wrapUp.
	logger detach! !

!MECCGModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/10/2016 15:24'!
test_01_SimpleUsage
	"Modelar empezar un juego con 2 jugadores que empieza (Game Setup),
	se juegan turnos y termina (Victory Condition)"

	| result |
	match := Match of: MiddleEarthCardGame rules.
	match addPlayer: (MECCGPlayer named: 'Fede Padawan').
	match addPlayer: (MECCGPlayer named: 'Pieter').
	match board: MECCGBoard new.
	self assert: match isReadyForSetup.
	match setup.
	self assert: match isReadyForPlay.
	match play.
	match wrapUp.
	result := match result.
! !

!MECCGModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 11:31'!
test_02_PlayerOrder
	"We test that the players go in the correct order."

	self playMatch.
	self checkLogger: logger toMakeSureThePlayerOrderIsTheSameAsIn: players! !

!MECCGModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 11:22'!
test_03_LastActionOfThePhase
	"We test that, for each phase, the last action taken is the 'EndPhase' action."

	self playMatch.
	self checkThatLastActionPlayedInEachPhaseIsEndPhaseInLogger: logger.! !

!MECCGModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 11:22'!
test_04_PhaseOrder
	"We test that the phases of the turn occur in the right order."

	self playMatch.
	self checkLogger: logger toMakeSureThePhaseOrderIsTheSameAsIn: match rules turnPhases.! !

!MECCGModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/10/2016 15:25'!
test_05_MatchSetup

	match := Match of: MiddleEarthCardGame rules.
	
	self deny: match isReadyForSetup.
	match addPlayer: (MECCGPlayer named: 'Fede Padawan').
	self deny: match isReadyForSetup.
	match addPlayer: (MECCGPlayer named: 'Pieter').
	self deny: match isReadyForSetup.
	match board: MECCGBoard new.
	self assert: match isReadyForSetup.
	match setup.! !

!MECCGModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/10/2016 15:25'!
test_06_ResultEvolution
	"Modelar empezar un juego con N jugadores que empieza (Game Setup), 
	se juegan turnos y termina (Victory Condition)"

	| result |
	match := Match of: MiddleEarthCardGame rules.
	match addPlayer: (MECCGPlayer named: 'Fede Padawan').
	match addPlayer: (MECCGPlayer named: 'Pieter').
	match board: MECCGBoard new.
	self assert: match isReadyForSetup.
	self assert: match result isNil.
	match setup.
	self assert: match result notNil.
	self assert: match result roundsFullyPlayed isNil.
	self assert: match isReadyForPlay.
	match play.
	match wrapUp.
	result := match result.
	self assert: result roundsFullyPlayed notNil.
! !

!MECCGModelUsageExamples methodsFor: 'public' stamp: 'SergioFedi 5/10/2016 15:23'!
test_07_LoggingAMatch
	"Modelar empezar un juego con N jugadores que empieza (Game Setup), 
	se juegan turnos y termina (Victory Condition)"

	| result |
	match := Match of: MiddleEarthCardGame rules.
	logger := MECCGTranscriptLogger on: match.
	logger logEvents: #somethingHappened.
	logger logOn: Transcript.
	match addPlayer: (MECCGPlayer named: 'Fede Padawan').
	match addPlayer: (MECCGPlayer named: 'Pieter').
	match board: MECCGBoard new.
	self assert: match isReadyForSetup.
	match setup.
	self assert: match isReadyForPlay.
	match play.
	match wrapUp.
	result := match result.
	logger detach.! !

TestCase subclass: #MECCGTranscriptLoggerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support-Tests'!

!MECCGTranscriptLoggerTest commentStamp: '<historical>' prior: 0!
A MECCGTranscriptLoggerTest is a test class for testing the behavior of MECCGTranscriptLogger!

!MECCGTranscriptLoggerTest methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 14:44'!
test_01_Subscribing

	| logger |
	logger := MECCGTranscriptLogger on: self.
	self assert: logger source = self.
	self deny: (self hasActionsWithReceiver: logger).
	logger logEvents: #testEvent.
	self assert: (self hasActionsWithReceiver: logger).
	logger detach.
	self deny: (self hasActionsWithReceiver: logger).
! !

!MECCGTranscriptLoggerTest methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 15:00'!
test_02_LogIsNilAtCreation

	| logger |
	logger := MECCGTranscriptLogger on: self.
	self assert: (logger sink isNil).
	logger logOn: Transcript.
	self assert: (logger sink notNil).
! !

TestCase subclass: #MTGSModelUsageExamples
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model-Tests'!

!MTGSModelUsageExamples methodsFor: 'public' stamp: 'FedericoFigueredo 5/10/2016 09:04'!
test_01_SimpleUsage
	"Modelar empezar un juego con 2 jugadores que empieza (Game Setup), 
	se juegan turnos y termina (Victory Condition)"

	| result match logger |
	match := MTGSMatch of: MagicSolitaireCardGame rules.
	logger := MECCGLogger on: match.
	logger logEvents: #somethingHappened.
	match addEnemy: (MTGSPlayer named: 'Solitaire AI').
	match addPlayer: (MTGSHumanPlayer named: 'Sergio').
	match board: MTGSBoard new.
	self assert: match result isNil.
	match setup.
	self assert: match result notNil.
	self assert: match isReadyForPlay.
	self assert: match result roundsFullyPlayed isNil.
	match play.
	match wrapUp.
	result := match result.
	self assert: result roundsFullyPlayed notNil.
	logger detach.! !

TestCase subclass: #MTGSPlayerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model-Tests'!

!MTGSPlayerTest commentStamp: '<historical>' prior: 0!
A MTGSPlayerTest is a test class for testing the behavior of MTGSPlayer!

!MTGSPlayerTest methodsFor: 'as yet unclassified' stamp: 'SergioFedi 5/8/2016 19:29'!
test_01_Creation

	| player |
	player := MTGSPlayer new.
	player hitPoints > 0.! !

TestCase subclass: #MatchTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model-Tests'!

!MatchTest commentStamp: '<historical>' prior: 0!
A MatchTest is a test class for testing the behavior of Match!

!MatchTest methodsFor: 'public' stamp: 'SergioFedi 5/10/2016 15:12'!
test_01_PlayersKnowTheMatch

	| match |
	match := Match new.
	match addPlayer: (MECCGPlayer named: 'Fede Padawan').
	match addPlayer: (MECCGPlayer named: 'Pieter').
	self assert: (match players allSatisfy: [:player | player match = match]).! !

TestCase subclass: #StringOfObjectsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support-Tests'!

!StringOfObjectsTest methodsFor: 'public' stamp: 'SergioFedi 5/5/2016 07:00'!
test_01_Creation

	| stringOfObjects |
	stringOfObjects := StringOfObjects new.
	self assert: stringOfObjects size = 0.! !

!StringOfObjectsTest methodsFor: 'public' stamp: 'SergioFedi 5/5/2016 07:01'!
test_02_Add

	| stringOfObjects |
	stringOfObjects := StringOfObjects new.
	stringOfObjects add: Object new.
	self assert: stringOfObjects size = 1.! !

!StringOfObjectsTest methodsFor: 'public' stamp: 'SergioFedi 5/5/2016 07:16'!
test_03_AddAll

	| stringOfObjects |
	stringOfObjects := StringOfObjects new.
	stringOfObjects addAll: #(1 '' 3 Object).
	self assert: stringOfObjects size = 4.
	! !

!StringOfObjectsTest methodsFor: 'public' stamp: 'SergioFedi 5/5/2016 07:17'!
test_04_CreationWithAll

	| stringOfObjects |
	stringOfObjects := StringOfObjects withAll: #().
	self assert: stringOfObjects size = 0.
	stringOfObjects := StringOfObjects withAll: #(1 2 3 4 5 6).
	self assert: stringOfObjects size = 6.
	! !

!StringOfObjectsTest methodsFor: 'public' stamp: 'FedericoFigueredo 5/8/2016 13:30'!
test_05_EmptyStringIsAlwaysSubstring

	| stringOfObjects |
	stringOfObjects := StringOfObjects new.
	self assert: (stringOfObjects isSubstringOfInfiniteLoopGeneratedBy: #()).
	self assert: (stringOfObjects isSubstringOfInfiniteLoopGeneratedBy: #(1 2 3)).
	
		! !

!StringOfObjectsTest methodsFor: 'public' stamp: 'FedericoFigueredo 5/8/2016 13:34'!
test_06_ComplexCasesOfObjectsInOrder

	| stringOfObjects templateObjects |
	templateObjects := OrderedCollection withAll: #(1 2 3 4).
	stringOfObjects := StringOfObjects withAll: #(1 2 3).
	self assert: (stringOfObjects isSubstringOfInfiniteLoopGeneratedBy: templateObjects).
	stringOfObjects := StringOfObjects withAll: #(1 2 3 4 1).
	self assert: (stringOfObjects isSubstringOfInfiniteLoopGeneratedBy: templateObjects).
	stringOfObjects := StringOfObjects withAll: #(1 2 3 4 1 2 3 4 1 2).
	self assert: (stringOfObjects isSubstringOfInfiniteLoopGeneratedBy: templateObjects).
	! !

!StringOfObjectsTest methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 02:31'!
test_07_SimpleCasesOfStringsOrderedAllowingZeroOrMoreInstancesOfOneElement

	| strings stringOfObjects |
	strings := OrderedCollection with: 'abc' with: 'ABC' with: '123' with: 'xyz'.
	stringOfObjects := StringOfObjects new.
	self deny: (stringOfObjects areTheseStringsOrdered: strings allowingZeroOrMoreInstancesOf: 'ABC').
	stringOfObjects := StringOfObjects withAll: (OrderedCollection with: 'abc' with: '123' with: 'xyz').
	self assert: (stringOfObjects areTheseStringsOrdered: strings allowingZeroOrMoreInstancesOf: 'ABC').
	stringOfObjects := StringOfObjects withAll: (OrderedCollection with: 'abc' with: 'ABC' with: '123' with: 'xyz').
	self assert: (stringOfObjects areTheseStringsOrdered: strings allowingZeroOrMoreInstancesOf: 'ABC').
	stringOfObjects := StringOfObjects withAll: (OrderedCollection with: 'abc' with: 'ABC' with: 'ABC' with: '123' with: 'xyz').
	self assert: (stringOfObjects areTheseStringsOrdered: strings allowingZeroOrMoreInstancesOf: 'ABC').
! !

!StringOfObjectsTest methodsFor: 'public' stamp: 'FedericoFigueredo 5/8/2016 13:44'!
test_08_ComplexCasesOfStringsOrderedAllowingZeroOrMoreInstancesOfOneElement

	| regularExpression |
	"regExpForAreTheseStringsOrdered = giveMeTheRegularExpressionToCheckWhetherTheseCharactersAreOrdered"
	regularExpression := StringOfObjects new regExpForAreTheseCharactersOrdered: 'ABCD' allowingZeroOrMoreInstancesOf: $B.
	"matchesRegex = matchesRegularExpression"
	self assert: ('ACD' matchesRegex: regularExpression).
	self assert: ('ABCD' matchesRegex: regularExpression).
	self assert: ('ABBBCD' matchesRegex: regularExpression).
	self assert: ('ACDABCDABBBCD' matchesRegex: regularExpression).
	self deny: ('' matchesRegex: regularExpression).
	self deny: ('ABD' matchesRegex: regularExpression).
	self deny: ('AD' matchesRegex: regularExpression).
	self deny: ('ABB' matchesRegex: regularExpression).! !

Object subclass: #Action
	instanceVariableNames: 'actor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!Action commentStamp: 'SergioFedi 5/7/2016 11:13' prior: 0!
I model the action that a player takes during his turn.
!

!Action methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:07'!
actor
	^ actor! !

!Action methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:07'!
actor: anObject
	actor := anObject! !

!Action methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 09:13'!
name

	^'generic'.! !

!Action methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:06'!
perform
! !

Action subclass: #MatchAction
	instanceVariableNames: 'match'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MatchAction commentStamp: 'SergioFedi 5/8/2016 21:38' prior: 0!
I model the action that a player takes during his turn in a game.
!

MatchAction subclass: #MECCGAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MECCGAction commentStamp: 'SergioFedi 5/8/2016 21:42' prior: 0!
I model the action that a player takes during a match of Middle Earth Collectible Card Game.
!

MECCGAction subclass: #EndPhaseAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!EndPhaseAction commentStamp: 'SergioFedi 5/7/2016 11:13' prior: 0!
I model the action of ending a phase.!

!EndPhaseAction methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 09:10'!
endsPhase

	^true.! !

!EndPhaseAction methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 09:14'!
name

	^'EndPhase'.! !

MatchAction subclass: #MTGSAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MTGSAction commentStamp: 'SergioFedi 5/8/2016 21:42' prior: 0!
I model the action that a player takes during a match of Magic The Gathering Solitaire.!

MTGSAction subclass: #MTGSAttackAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MTGSAttackAction commentStamp: 'SergioFedi 5/8/2016 21:43' prior: 0!
I model the attack action that a player takes during a match of Magic The Gathering Solitaire.!

!MTGSAttackAction methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:49'!
perform

	self match enemy reduceHitpointsBy: 3.! !

!MatchAction class methodsFor: 'instance creation' stamp: 'SergioFedi 5/8/2016 21:40'!
match: aMatch

	^self new match: aMatch.! !

!MatchAction methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:39'!
endsPhase

	^false.! !

!MatchAction methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:39'!
match
	^ match! !

!MatchAction methodsFor: 'private' stamp: 'SergioFedi 5/8/2016 21:39'!
match: anObject
	match := anObject! !

Object subclass: #CardGame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!CardGame commentStamp: 'SergioFedi 5/8/2016 01:22' prior: 0!
I model a Card Game.
I can answer the rules of the game.!

!CardGame class methodsFor: 'instance creation' stamp: 'SergioFedi 5/8/2016 01:20'!
rules
	self subclassResponsibility! !

CardGame subclass: #MagicSolitaireCardGame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MagicSolitaireCardGame commentStamp: 'SergioFedi 5/8/2016 01:24' prior: 0!
I model a Solitaire variation of the Magic The Gathering Card Game.
I can answer the rules of the game.
!

!MagicSolitaireCardGame class methodsFor: 'instance creation' stamp: 'SergioFedi 5/8/2016 01:31'!
rules

	^MagicSolitaireCardGameRules new.! !

CardGame subclass: #MiddleEarthCardGame
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MiddleEarthCardGame commentStamp: 'SergioFedi 4/21/2016 08:31' prior: 0!
I model the game of Middle Earth Collectible Card Game.
I can answer the rules of the game.
!

!MiddleEarthCardGame class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:09'!
rules

	^MiddleEarthCardGameRules new.! !

Object subclass: #CardGameRules
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!CardGameRules commentStamp: 'SergioFedi 5/8/2016 01:29' prior: 0!
I model the rules of a Card Game.

My responsibility is to run matches of that game.
!

!CardGameRules methodsFor: 'private' stamp: 'SergioFedi 5/8/2016 01:42'!
incrementRoundCountOf: aMatch

	aMatch result roundsFullyPlayed: aMatch result roundsFullyPlayed + 1.! !

!CardGameRules methodsFor: 'private' stamp: 'SergioFedi 5/8/2016 01:41'!
initializeRoundCountOf: aMatch 

	aMatch result roundsFullyPlayed: 0.! !

!CardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 01:47'!
isMatchReady: aMatch

	^self subclassResponsibility.! !

!CardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 01:33'!
isMatchReadyForSetup: aMatch

	^self subclassResponsibility.! !

!CardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 01:48'!
play: aMatch

	^self subclassResponsibility.! !

!CardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 01:36'!
setupMatch: aMatch

	aMatch result: MatchOutcome new.
	^self.! !

!CardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 01:48'!
wrapUpMatch: aMatch
	
	^self subclassResponsibility.! !

CardGameRules subclass: #MagicSolitaireCardGameRules
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MagicSolitaireCardGameRules commentStamp: 'SergioFedi 5/8/2016 01:30' prior: 0!
I model the rules of Magic The Gathering Solitaire Card Game.

My responsibility is to run matches of that game.
!

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:15'!
isFinished: aMatch

	(aMatch player hitPoints <= 0) ifTrue: [^true].
	(aMatch enemy hitPoints <= 0) ifTrue: [^true].
	^false.! !

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 01:39'!
isMatchReady: aMatch

	^true.! !

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 01:34'!
isMatchReadyForSetup: aMatch

	^true.! !

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:48'!
performAction: anAction 

	anAction perform.! !

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:12'!
play: aMatch
	"Play the match until it is finished"

	self initializeRoundCountOf: aMatch.
	[self isFinished: aMatch] whileFalse: [
			self playRoundOf: aMatch.
			self incrementRoundCountOf: aMatch.
	].! !

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:45'!
playPhase: phase ofPlayer: aPlayer
	| keepPlayingPlayerActions |
	aPlayer match beginPhase: phase ofPlayer: aPlayer.
	phase beginIn: aPlayer match.
	keepPlayingPlayerActions := true.
	[ keepPlayingPlayerActions ]
		whileTrue: [ 
			| chosenAction |
			chosenAction := aPlayer chooseActionAmong: (phase availableActionsForPlayer: aPlayer).
			self performAction: chosenAction.
			aPlayer match player: aPlayer played: chosenAction.
			keepPlayingPlayerActions := chosenAction endsPhase not ].
	phase endIn: aPlayer match.
	^ aPlayer match endPhase: phase ofPlayer: aPlayer! !

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:36'!
playRoundOf: aMatch
	aMatch players
		do: [ :player | 
			self playTurnOf: player.
			(self isFinished: aMatch)
				ifTrue: [ ^ self ] ]! !

!MagicSolitaireCardGameRules methodsFor: 'private' stamp: 'SergioFedi 5/8/2016 19:38'!
playTurnOf: aPlayer
	aPlayer match beginTurnOf: aPlayer.
	self turnPhases do: [ :phase | self playPhase: phase ofPlayer: aPlayer ].
	aPlayer match endTurnOf: aPlayer! !

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:36'!
turnPhases

	^Array
		with: (MTGSPhase named: 'Untap')
		with: (MTGSPhase named: 'Draw')
		with: (MTGSPhase named: 'Main 1')
		with: (MTGSPhase named: 'Attack')
		with: (MTGSPhase named: 'Main 2')
		with: (MTGSPhase named: 'End of Turn').! !

!MagicSolitaireCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 01:39'!
wrapUpMatch: aMatch
	! !

CardGameRules subclass: #MiddleEarthCardGameRules
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MiddleEarthCardGameRules commentStamp: 'SergioFedi 4/21/2016 08:36' prior: 0!
I model the rules of the Middle Earth Card Game.

My responsibility is to run matches of that game.
!

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 09:01'!
isFinished: aMatch

	^aMatch result isFinished.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:34'!
isMatchReady: aMatch

	^true.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'FedericoFigueredo 5/10/2016 09:11'!
isMatchReadyForSetup: aMatch

	(aMatch players size ~= 2) ifTrue: [^ false].
	(self isValidBoard: aMatch board inMatch: aMatch) ifFalse: [^ false].
	^ true.! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'FedericoFigueredo 5/10/2016 09:18'!
isValidBoard: aMECCGBoard inMatch: aMatch 
	| validator |
	(aMatch board isNil) ifTrue: [^ false].
	validator := MECCGBoardValidator new.
	^ validator isValid: aMECCGBoard.! !

!MiddleEarthCardGameRules methodsFor: 'private' stamp: 'FedericoFigueredo 5/5/2016 09:18'!
performAction: anAction ! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/25/2016 12:26'!
play: aMatch
	"Play the match until it is finished"

	self initializeRoundCountOf: aMatch.
	[self isFinished: aMatch] whileFalse: [
			self playRoundOf: aMatch.
			self incrementRoundCountOf: aMatch.
	].
! !

!MiddleEarthCardGameRules methodsFor: 'private' stamp: 'SergioFedi 5/8/2016 21:45'!
playPhase: phase ofPlayer: aPlayer
	| keepPlayingPlayerActions |
	aPlayer match beginPhase: phase ofPlayer: aPlayer.
	phase beginIn: aPlayer match.
	keepPlayingPlayerActions := true.
	[ keepPlayingPlayerActions ]
		whileTrue: [ 
			| chosenAction |
			chosenAction := aPlayer chooseActionAmong: (phase availableActionsForPlayer: aPlayer).
			self performAction: chosenAction.
			aPlayer match player: aPlayer played: chosenAction.
			keepPlayingPlayerActions := chosenAction endsPhase not ].
	phase endIn: aPlayer match.
	^ aPlayer match endPhase: phase ofPlayer: aPlayer! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:36'!
playRoundOf: aMatch
	aMatch players
		do: [ :player | 
			self playTurnOf: player.
			(self isFinished: aMatch)
				ifTrue: [ ^ self ] ]! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:37'!
playTurnOf: aPlayer
	aPlayer match beginTurnOf: aPlayer.
	self turnPhases do: [ :phase | self playPhase: phase ofPlayer: aPlayer ].
	aPlayer match endTurnOf: aPlayer! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'FedericoFigueredo 5/6/2016 09:04'!
turnPhases

	^Array
		with: (MECCGPhase named: 'Untap')
		with: (MECCGPhase named: 'Organization')
		with: (MECCGPhase named: 'Long Event')
		with: (MECCGPhase named: 'Movement / Hazard')
		with: (MECCGPhase named: 'Site')
		with: (EndOfTurnPhase named: 'End of Turn').
		! !

!MiddleEarthCardGameRules methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:57'!
wrapUpMatch: aMatch

	^self.! !

Object subclass: #GameBoard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!GameBoard commentStamp: 'SergioFedi 5/8/2016 14:47' prior: 0!
I model the board of a game.!

GameBoard subclass: #MECCGBoard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MECCGBoard commentStamp: 'SergioFedi 5/8/2016 14:47' prior: 0!
I model the board of a Middle Earth Collectible Card Game.!

GameBoard subclass: #MTGSBoard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MTGSBoard commentStamp: 'SergioFedi 5/8/2016 14:48' prior: 0!
I model the board of a Magic The Gathering Solitaire Game.!

Object subclass: #MECCGListener
	instanceVariableNames: 'source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support'!

!MECCGListener commentStamp: 'SergioFedi 4/29/2016 09:46' prior: 0!
I am a Listener (as in the Observer pattern) of MECCG Games, Matches, Players, etc.
!

!MECCGListener class methodsFor: 'instance creation' stamp: 'SergioFedi 4/29/2016 09:55'!
on: anObject

	^self new source: anObject.! !

!MECCGListener methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:49'!
detach

	self source removeActionsWithReceiver: self.! !

!MECCGListener methodsFor: 'private' stamp: 'SergioFedi 4/29/2016 09:45'!
logEventOccurence: aString
	self subclassResponsibility! !

!MECCGListener methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:49'!
logEvents: aSymbol

	self source 
		when: aSymbol
		send: #logEventOccurence:
		to: self.! !

!MECCGListener methodsFor: 'public' stamp: 'SergioFedi 4/29/2016 09:49'!
source
	^ source! !

!MECCGListener methodsFor: 'private' stamp: 'SergioFedi 4/29/2016 09:49'!
source: anObject
	source := anObject! !

MECCGListener subclass: #MECCGLogger
	instanceVariableNames: 'log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support'!

!MECCGLogger commentStamp: 'SergioFedi 4/29/2016 09:51' prior: 0!
I listen to events of a MECCG and log them in a collection.!

!MECCGLogger methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 08:29'!
areTheseMessagesOrdered: aSequenceableCollection

	^self log isSubstringOfInfiniteLoopGeneratedBy: aSequenceableCollection.! !

!MECCGLogger methodsFor: 'public' stamp: 'SergioFedi 5/3/2016 11:43'!
areTheseMessagesOrdered: aCollection allowingMultipleInstancesOf: aString

	^self log areTheseStringsOrdered: aCollection allowingZeroOrMoreInstancesOf: aString.! !

!MECCGLogger methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 11:10'!
initialize

	self log: StringOfObjects new.! !

!MECCGLogger methodsFor: 'private' stamp: 'SergioFedi 4/29/2016 09:53'!
log
	^ log! !

!MECCGLogger methodsFor: 'private' stamp: 'SergioFedi 4/29/2016 09:53'!
log: anObject
	log := anObject! !

!MECCGLogger methodsFor: 'private' stamp: 'SergioFedi 4/29/2016 09:51'!
logEventOccurence: aString

	self log add: aString.! !

!MECCGLogger methodsFor: 'public' stamp: 'SergioFedi 5/2/2016 12:09'!
logSize

	^self log size.! !

MECCGListener subclass: #MECCGTranscriptLogger
	instanceVariableNames: 'sink'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support'!

!MECCGTranscriptLogger commentStamp: 'SergioFedi 4/28/2016 16:58' prior: 0!
This class listens other objects and logs the events that are of interest.!

!MECCGTranscriptLogger methodsFor: 'private' stamp: 'SergioFedi 4/29/2016 09:51'!
logEventOccurence: aString

	self sink nextPutAll: aString.
	self sink cr.
	self sink flush.! !

!MECCGTranscriptLogger methodsFor: 'public' stamp: 'SergioFedi 4/28/2016 19:55'!
logOn: aThreadSafeTranscript

	self sink: aThreadSafeTranscript.! !

!MECCGTranscriptLogger methodsFor: 'private' stamp: 'SergioFedi 4/28/2016 19:48'!
sink
	^ sink! !

!MECCGTranscriptLogger methodsFor: 'private' stamp: 'SergioFedi 4/28/2016 19:48'!
sink: anObject
	sink := anObject! !

Object subclass: #Match
	instanceVariableNames: 'rules players result board'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!Match commentStamp: 'SergioFedi 4/21/2016 08:22' prior: 0!
I model a match of a game.

You can see how to use me in ModelUsageExamples
!

Match subclass: #MTGSMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MTGSMatch commentStamp: 'SergioFedi 5/8/2016 19:17' prior: 0!
I model a match of Magic The Gathering Solitaire Game.

!

!MTGSMatch methodsFor: 'private' stamp: 'SergioFedi 5/8/2016 19:23'!
addEnemy: aPlayer

	enemy := aPlayer.! !

!MTGSMatch methodsFor: 'private' stamp: 'SergioFedi 5/8/2016 19:39'!
addPlayer: aPlayer

	players := OrderedCollection new.
	super addPlayer: aPlayer.! !

!MTGSMatch methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:19'!
addPlayers: aCollectionOfPlayers

	self shouldNotImplement.! !

!MTGSMatch methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:30'!
enemy
	^ enemy! !

!MTGSMatch methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:25'!
player

	^players first.! !

!Match class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:12'!
of: matchRules

	^self new
		rules: matchRules.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/25/2016 08:30'!
addPlayer: aPlayer

	aPlayer match: self.
	players add: aPlayer.! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/2/2016 08:44'!
addPlayers: aCollectionOfPlayers 
	aCollectionOfPlayers do: [ :player | 
		self addPlayer: player].! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 08:53'!
beginPhase: phase ofPlayer: aPlayerOfMECCG 
	self triggerEvent: #somethingHappened with: 'Begin phase named ', phase name, ' of player ', aPlayerOfMECCG name.! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 08:49'!
beginTurnOf: aPlayerOfMECCG 
	self triggerEvent: #somethingHappened with: 'Begin turn of player ', aPlayerOfMECCG name.! !

!Match methodsFor: 'accessing' stamp: 'SergioFedi 5/8/2016 14:54'!
board
	^ board! !

!Match methodsFor: 'accessing' stamp: 'SergioFedi 5/8/2016 14:54'!
board: aGameBoard
	board := aGameBoard! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 08:55'!
endPhase: phase ofPlayer: aPlayerOfMECCG 
	self triggerEvent: #somethingHappened with: 'End of phase named ', phase name, ' of player ', aPlayerOfMECCG name.! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 09:01'!
endTurnOf: aPlayerOfMECCG 
	self triggerEvent: #somethingHappened with: 'End turn of player ', aPlayerOfMECCG name.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:19'!
initialize
	super initialize.
	players := OrderedCollection new.! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/10/2016 09:04'!
isReadyForPlay

	^self rules isMatchReady: self.! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/10/2016 09:03'!
isReadyForSetup
	^ self rules isMatchReadyForSetup: self! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/28/2016 22:38'!
play

	self triggerEvent: #somethingHappened with: 'Match started'.
	self rules play: self.
! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/5/2016 09:13'!
player: aPlayerOfMECCG played: anAction

	self triggerEvent: #somethingHappened with: 'Action ', anAction name, ' played by player ', aPlayerOfMECCG name.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 08:48'!
players
	^ players! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:40'!
result

	^result.! !

!Match methodsFor: 'private' stamp: 'SergioFedi 4/17/2016 17:49'!
result: aMatchResult

	result := aMatchResult.! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:13'!
rules
	^ rules! !

!Match methodsFor: 'private' stamp: 'SergioFedi 4/17/2016 17:13'!
rules: anObject
	rules := anObject! !

!Match methodsFor: 'public' stamp: 'FedericoFigueredo 5/10/2016 09:03'!
setup
	self isReadyForSetup
		ifTrue: [ self rules setupMatch: self ]! !

!Match methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:57'!
wrapUp

	self rules wrapUpMatch: self.! !

Object subclass: #MatchOutcome
	instanceVariableNames: 'matchStatistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MatchOutcome commentStamp: 'SergioFedi 4/21/2016 08:27' prior: 0!
I model the outcome of a match.

The particular data I will contain will be determined by the rules of the match.
!

!MatchOutcome methodsFor: 'as yet unclassified' stamp: 'FedericoFigueredo 5/3/2016 08:30'!
initialize
	super initialize.
	matchStatistics := Dictionary new.
	self roundsFullyPlayed: nil.
	self isFinished: false.! !

!MatchOutcome methodsFor: 'accesing' stamp: 'SergioFedi 4/22/2016 08:58'!
isFinished

	^matchStatistics at: 'isFinished'.! !

!MatchOutcome methodsFor: 'public' stamp: 'SergioFedi 4/22/2016 08:58'!
isFinished: anObject 

	matchStatistics at: 'isFinished' put: anObject.! !

!MatchOutcome methodsFor: 'accesing' stamp: 'FedericoFigueredo 5/3/2016 08:31'!
roundsFullyPlayed

	^matchStatistics at: #roundsFullyPlayed.! !

!MatchOutcome methodsFor: 'public' stamp: 'FedericoFigueredo 5/3/2016 08:46'!
roundsFullyPlayed: anObject 

	matchStatistics at: #roundsFullyPlayed put: anObject.! !

Object subclass: #Player
	instanceVariableNames: 'name match'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!Player commentStamp: 'SergioFedi 4/21/2016 08:30' prior: 0!
I model a participant of a Match that plays in it.
!

Player subclass: #MECCGPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MECCGPlayer commentStamp: 'SergioFedi 4/25/2016 08:36' prior: 0!
I model a participant of a Match of MiddleEarthCardGame that plays in it.!

!MECCGPlayer methodsFor: 'public' stamp: 'SergioFedi 4/26/2016 09:09'!
chooseActionAmong: aCollection

	^aCollection atRandom.! !

Player subclass: #MTGSPlayer
	instanceVariableNames: 'hitPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MTGSPlayer commentStamp: 'SergioFedi 5/8/2016 19:27' prior: 0!
I model a participant of a Match of MagicTheGatheringSolitareGame that plays in it.!

MTGSPlayer subclass: #MTGSHumanPlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MTGSHumanPlayer commentStamp: 'SergioFedi 5/8/2016 19:10' prior: 0!
I model a human player that plays a game of Magic The Gathering Solitaire.

Whenever the game needs something to be done by the player, I ask him what to do through an interface.!

!MTGSHumanPlayer methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:52'!
chooseActionAmong: aCollection

	^aCollection atRandom.
	
	"^MenuMorph
		chooseFrom: (aCollection collect: [:each | each printString])
		values: aCollection
		lines: #()
		title: 'Choose your action'."! !

!MTGSPlayer methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 19:27'!
hitPoints

	^hitPoints.! !

!MTGSPlayer methodsFor: 'initialization' stamp: 'SergioFedi 5/8/2016 19:30'!
initialize

	super initialize.

	hitPoints := 20.! !

!MTGSPlayer methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:51'!
reduceHitpointsBy: anInteger

	hitPoints := hitPoints - anInteger.! !

!Player class methodsFor: 'instance creation' stamp: 'SergioFedi 4/17/2016 17:16'!
named: aString

	^self new name: aString.! !

!Player methodsFor: 'public' stamp: 'SergioFedi 4/25/2016 08:30'!
match
	^ match! !

!Player methodsFor: 'public' stamp: 'SergioFedi 4/25/2016 08:30'!
match: anObject
	match := anObject! !

!Player methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:15'!
name
	^ name! !

!Player methodsFor: 'public' stamp: 'SergioFedi 4/17/2016 17:15'!
name: anObject
	name := anObject! !

Object subclass: #StringOfObjects
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Support'!

!StringOfObjects commentStamp: 'SergioFedi 5/2/2016 11:10' prior: 0!
I generalize the concept of String.

From String of Characters to a String of Objects.

For the moment my purpose is to solve the pattern matching problem for the more general case of matching a pattern of objects instead of a pattern of characters.
!

!StringOfObjects class methodsFor: 'public' stamp: 'SergioFedi 5/5/2016 07:18'!
withAll: aCollection

	^(self new)
		addAll: aCollection;
		yourself.! !

!StringOfObjects methodsFor: 'public' stamp: 'SergioFedi 5/2/2016 11:26'!
add: anObject
 
	objects add: anObject.! !

!StringOfObjects methodsFor: 'public' stamp: 'SergioFedi 5/5/2016 07:17'!
addAll: aCollection

	aCollection do: [:each |
		self add: each.
	].! !

!StringOfObjects methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 13:51'!
alphabetFor: aCollection

	^(1 to: aCollection size) collect: [ :n | Character value: 64 + n ].! !

!StringOfObjects methodsFor: 'public' stamp: 'FedericoFigueredo 5/8/2016 13:44'!
areTheseStringsOrdered: aCollection allowingZeroOrMoreInstancesOf: aString

	| areOrdered logString regularExpresion |
	logString := self logAsStringFor: aCollection.
	regularExpresion := self regExpForAreTheseCharactersOrdered: aCollection allowingZeroOrMoreInstancesOf: aString.
	areOrdered := logString matchesRegex: regularExpresion.
	^areOrdered.! !

!StringOfObjects methodsFor: 'private' stamp: 'SergioFedi 5/2/2016 11:26'!
initialize

	super initialize.
	objects := OrderedCollection new.! !

!StringOfObjects methodsFor: 'public' stamp: 'SergioFedi 5/7/2016 14:40'!
isEmpty

	^objects isEmpty.! !

!StringOfObjects methodsFor: 'public' stamp: 'SergioFedi 5/6/2016 14:04'!
isSubstringOfInfiniteLoopGeneratedBy: aCollection
	"Answers true if aCollection of objects appear in order within the receiver.
	The presence of the objects is optional, but if they appear, they must be in the stated order."

	| areOrdered logString regularExpresion |
	aCollection isEmpty ifTrue: [ ^true ].
	logString := self logAsStringFor: aCollection.
	regularExpresion := self objectsAppearInOrderRegExpFor: aCollection.
	areOrdered := logString matchesRegex: regularExpresion.
	^areOrdered.! !

!StringOfObjects methodsFor: 'private' stamp: 'SergioFedi 5/6/2016 14:06'!
logAsStringFor: aCollection
	"Converts objects into a string consisting only of the interesting objects (aCollection)"

	| map string |
	string := WriteStream on: String new.
	map := Dictionary new.
	(self alphabetFor: aCollection) with: aCollection do: [ :character :object |
		map at: object put: character.
	].
	objects do: [ :object |
		| matchingObject |
		matchingObject := aCollection detect: [:obj | object asString beginsWith: obj asString] ifNone: [nil].
		(map includesKey: matchingObject) ifTrue: [ 
			string nextPut: (map at: matchingObject).
		].
	].
	^string contents.
	! !

!StringOfObjects methodsFor: 'private' stamp: 'SergioFedi 5/6/2016 12:26'!
objectsAppearInOrderRegExpFor: aCollection
	"ABCD -> (ABCD)*(A(B(C(D)?)?)?)?"

	| regExp codes |
	codes := self alphabetFor: aCollection.
	regExp := WriteStream on: String new.
	regExp nextPut: $(.
	regExp nextPutAll: codes.
	regExp nextPut: $).
	regExp nextPut: $*.
	codes do: [ :code |
		regExp nextPut: $(.
		regExp nextPut: code.
	].
	codes size timesRepeat: [ regExp nextPutAll: ')?' ].

	^regExp contents.
	! !

!StringOfObjects methodsFor: 'private' stamp: 'FedericoFigueredo 5/8/2016 13:47'!
regExpForAreTheseCharactersOrdered: aString allowingZeroOrMoreInstancesOf: aCharacter

	| regExp |
	self assert: aString size = 4.
	self assert: aString second = aCharacter.
	regExp := '(AB*CD)+'.
	^regExp.
	
! !

!StringOfObjects methodsFor: 'public' stamp: 'SergioFedi 5/2/2016 11:36'!
size

	^objects size.! !

Object subclass: #TurnPhase
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!TurnPhase commentStamp: 'SergioFedi 5/8/2016 21:32' prior: 0!
I model a phase of a turn.
A Phase of a Turn is a segment of a turn in which certain aspects of the game are resolved.

Examples:
	The beginning of the turn of a player is usually a phase of that player's turn.
	The section of a player's turn where he take the main actions is usually the Main Phase.
	
Of course all of this depends on the particular game.
!

TurnPhase subclass: #MECCGPhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MECCGPhase commentStamp: 'SergioFedi 4/26/2016 08:56' prior: 0!
I model a phase of a turn of the Middle Earth Collectible Card Game.
!

MECCGPhase subclass: #EndOfTurnPhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!EndOfTurnPhase commentStamp: 'SergioFedi 5/7/2016 11:12' prior: 0!
I model the end of turn phase of a turn of the Middle Earth Collectible Card Game.

For now, I check if the match has finished.!

!EndOfTurnPhase methodsFor: 'public' stamp: 'FedericoFigueredo 5/6/2016 09:06'!
endIn: aMatch
	(aMatch result roundsFullyPlayed >= 3) ifTrue: [ 
		aMatch result isFinished: true.
	]! !

!MECCGPhase methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:52'!
availableActionsForPlayer: aPlayerOfMECCG

	^Array
		with: MatchAction new 
		with: EndPhaseAction new.! !

TurnPhase subclass: #MTGSPhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MTGSPhase commentStamp: 'SergioFedi 5/8/2016 21:35' prior: 0!
I model a phase of a turn of the Magic The Gathering Solitaire Game.
!

!MTGSPhase methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:47'!
availableActionsForPlayer: aMECCGPlayer

	^Array
		with: (MTGSAttackAction match: aMECCGPlayer match) 
		with: EndPhaseAction new.! !

!TurnPhase class methodsFor: 'instance creation' stamp: 'SergioFedi 5/8/2016 21:50'!
named: aString

	^self new name: aString.! !

!TurnPhase methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:45'!
availableActionsForPlayer: aPlayerOfMECCG

	self subclassResponsibility.! !

!TurnPhase methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:33'!
beginIn: aMatch
	! !

!TurnPhase methodsFor: 'public' stamp: 'SergioFedi 5/8/2016 21:33'!
endIn: aMatch
	! !

!TurnPhase methodsFor: 'accessing' stamp: 'SergioFedi 5/8/2016 21:32'!
name
	^ name! !

!TurnPhase methodsFor: 'accessing' stamp: 'SergioFedi 5/8/2016 21:33'!
name: aString
	name := aString! !

Object subclass: #Validator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

Validator subclass: #MECCGBoardValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MECCG-Model'!

!MECCGBoardValidator methodsFor: 'public' stamp: 'FedericoFigueredo 5/10/2016 08:48'!
isValid: aMECCGBoard
	^ true.! !
